@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -24,12 +24,19 @@
 #include <chrono>
 #include <cstring>
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+// I prefer to use SQLITE_TRANSIENT instead of SQLITE_STATIC on bind calls
+// because when using SQLITE_STATIC in bind functions, we must keep the pointers
+// passed to it valid until calling sqlite3_step. SQLITE_TRANSIENT allocates
+// and copies data in bind functions.
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+
 namespace mobius
 {
 namespace database
 {
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief implementation struct
+//! \brief Implementation struct
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 struct statement::impl
 {
@@ -40,7 +47,7 @@ struct statement::impl
 };
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief destroy shared implementation
+//! \brief Destroy shared implementation
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 statement::impl::~impl ()
 {
@@ -49,7 +56,7 @@ statement::impl::~impl ()
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief create statement object
+//! \brief Create statement object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 statement::statement ()
 {
@@ -67,8 +74,20 @@ statement::statement (database db, sqlite3_stmt *stmt)
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief bind int value
-//! \param idx value index
+//! \brief Bind bool value
+//! \param idx Value index
+//! \param value value
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+statement::bind (int idx, bool value)
+{
+  if (sqlite3_bind_int (impl_->stmt, idx, value ? 1 : 0) != SQLITE_OK)
+    throw std::runtime_error (MOBIUS_EXCEPTION_SQLITE);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Bind int value
+//! \param idx Value index
 //! \param value value
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
@@ -79,8 +98,8 @@ statement::bind (int idx, int value)
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief bind uint64_t value
-//! \param idx value index
+//! \brief Bind uint64_t value
+//! \param idx Value index
 //! \param value value
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
@@ -91,9 +110,9 @@ statement::bind (int idx, std::int64_t value)
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief bind double value
-//! \param idx value index
-//! \param v value
+//! \brief Bind double value
+//! \param idx Value index
+//! \param v Value
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
 statement::bind (int idx, double value)
@@ -103,38 +122,38 @@ statement::bind (int idx, double value)
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief bind C string value
-//! \param idx value index
-//! \param v value
+//! \brief Bind C string value
+//! \param idx Value index
+//! \param v Value
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
 statement::bind (int idx, const char *value)
 {
-  if (sqlite3_bind_text (impl_->stmt, idx, value, strlen (value), SQLITE_STATIC) != SQLITE_OK)
+  if (sqlite3_bind_text (impl_->stmt, idx, value, strlen (value), SQLITE_TRANSIENT) != SQLITE_OK)
     throw std::runtime_error (MOBIUS_EXCEPTION_SQLITE);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief bind string value
-//! \param idx value index
-//! \param v value
+//! \brief Bind string lvalue
+//! \param idx Value index
+//! \param v Value
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
 statement::bind (int idx, const std::string& value)
 {
-  if (sqlite3_bind_text (impl_->stmt, idx, value.c_str (), value.length (), SQLITE_STATIC) != SQLITE_OK)
+  if (sqlite3_bind_text (impl_->stmt, idx, value.c_str (), value.length (), SQLITE_TRANSIENT) != SQLITE_OK)
     throw std::runtime_error (MOBIUS_EXCEPTION_SQLITE);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Bind bytearray value
-//! \param idx value index
-//! \param v value
+//! \brief Bind bytearray lvalue
+//! \param idx Value index
+//! \param v Value
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
 statement::bind (int idx, const mobius::bytearray& value)
 {
-  if (sqlite3_bind_blob (impl_->stmt, idx, value.begin (), value.size (), SQLITE_STATIC) != SQLITE_OK)
+  if (sqlite3_bind_blob (impl_->stmt, idx, value.begin (), value.size (), SQLITE_TRANSIENT) != SQLITE_OK)
     throw std::runtime_error (MOBIUS_EXCEPTION_SQLITE);
 }
 
@@ -146,15 +165,21 @@ statement::bind (int idx, const mobius::bytearray& value)
 void
 statement::bind (int idx, const mobius::datetime::datetime& value)
 {
-  auto timestamp = mobius::datetime::datetime_to_iso_string (value);
+  if (value)
+    {
+      auto timestamp = to_string (value);
 
-  if (sqlite3_bind_text (impl_->stmt, idx, timestamp.c_str (), timestamp.length (), SQLITE_STATIC) != SQLITE_OK)
-    throw std::runtime_error (MOBIUS_EXCEPTION_SQLITE);
+      if (sqlite3_bind_text (impl_->stmt, idx, timestamp.c_str (), timestamp.length (), SQLITE_TRANSIENT) != SQLITE_OK)
+        throw std::runtime_error (MOBIUS_EXCEPTION_SQLITE);
+    }
+
+  else
+    bind_null (idx);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Bind null value
-//! \param idx value index
+//! \param idx Value index
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
 statement::bind_null (int idx)
@@ -215,7 +240,7 @@ statement::get_column_count ()
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Check whether column value is null
-//! \param idx value index
+//! \param idx Value index
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 bool
 statement::is_column_null (int idx)
@@ -223,9 +248,19 @@ statement::is_column_null (int idx)
   return sqlite3_column_type (impl_->stmt, idx) == SQLITE_NULL;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get bool column value
+//! \param idx Value index
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+bool
+statement::get_column_bool (int idx)
+{
+  return sqlite3_column_int (impl_->stmt, idx) == 1;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Get int column value
-//! \param idx value index
+//! \param idx Value index
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 int
 statement::get_column_int (int idx)
@@ -235,7 +270,7 @@ statement::get_column_int (int idx)
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Get int64_t column value
-//! \param idx value index
+//! \param idx Value index
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 std::int64_t
 statement::get_column_int64 (int idx)
@@ -245,7 +280,7 @@ statement::get_column_int64 (int idx)
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Get double column value
-//! \param idx value index
+//! \param idx Value index
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 double
 statement::get_column_double (int idx)
@@ -255,7 +290,7 @@ statement::get_column_double (int idx)
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Get string column value
-//! \param idx value index
+//! \param idx Value index
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 std::string
 statement::get_column_string (int idx)
@@ -271,7 +306,7 @@ statement::get_column_string (int idx)
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Get bytearray column value
-//! \param idx value index
+//! \param idx Value index
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 mobius::bytearray
 statement::get_column_bytearray (int idx)
@@ -289,7 +324,7 @@ statement::get_column_bytearray (int idx)
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Get datetime column value
-//! \param idx value index
+//! \param idx Value index
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 mobius::datetime::datetime
 statement::get_column_datetime (int idx)
