@@ -24,6 +24,8 @@
 #include "folder.h"
 #include "file.h"
 #include "entry.h"
+#include <mobius/exception.inc>
+#include <stdexcept>
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Check if object type is <i>folder</i>
@@ -70,29 +72,12 @@ PyMobius_IO_Folder_from_cpp (mobius::io::folder obj)
 mobius::io::folder
 PyMobius_IO_Folder_as_cpp (PyObject *pyobj)
 {
+  if (!PyObject_IsInstance (pyobj, (PyObject *) &io_folder_t))
+    throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_MSG ("object type must be mobius.io.folder"));
+
   return * (reinterpret_cast <io_folder_o *>(pyobj)->obj);
 }
 
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief url getter
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static PyObject *
-tp_getter_url (io_folder_o *self)
-{
-  PyObject *ret = nullptr;
-
-  try
-    {
-      ret = mobius::py::pystring_from_std_string (self->obj->get_url ());
-    }
-  catch (const std::exception& e)
-    {
-      PyErr_SetString (PyExc_Exception, e.what ());
-    }
-
-  return ret;
-}
-
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief name getter
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -378,12 +363,6 @@ tp_getter_backup_time (io_folder_o *self)
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static PyGetSetDef tp_getset[] =
 {
-  {
-    (char *) "url",
-    (getter) tp_getter_url,
-    (setter) 0,
-    (char *) "URL as string", NULL
-  },
   {
     (char *) "name",
     (getter) tp_getter_name,
@@ -571,6 +550,26 @@ tp_f_is_browseable (io_folder_o *self)
   return ret;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief get_extension method
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_get_extension (io_folder_o *self)
+{
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pystring_from_std_string (self->obj->get_extension ());
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief get_parent method
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -730,6 +729,103 @@ tp_f_clear (io_folder_o *self, PyObject *)
   return ret;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief reload method
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_reload (io_folder_o *self, PyObject *)
+{
+  PyObject *ret = nullptr;
+
+  try
+    {
+      self->obj->reload ();
+
+      Py_INCREF (Py_None);
+      ret = Py_None;
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief copy method
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_copy (io_folder_o *self, PyObject *args)
+{
+  // parse input args
+  mobius::io::folder arg_dst;
+
+  try
+    {
+      arg_dst = PyMobius_IO_Folder_as_cpp (mobius::py::get_arg (args, 0));
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_ValueError, e.what ());
+      return nullptr;
+    }
+
+  // execute C++ code
+  PyObject *ret = nullptr;
+
+  try
+    {
+      self->obj->copy (arg_dst);
+
+      Py_INCREF (Py_None);
+      ret = Py_None;
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief move method
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_move (io_folder_o *self, PyObject *args)
+{
+  // parse input args
+  mobius::io::folder arg_dst;
+
+  try
+    {
+      arg_dst = PyMobius_IO_Folder_as_cpp (mobius::py::get_arg (args, 0));
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_ValueError, e.what ());
+      return nullptr;
+    }
+
+  // execute C++ code
+  PyObject *ret = nullptr;
+
+  try
+    {
+      self->obj->move (arg_dst);
+
+      Py_INCREF (Py_None);
+      ret = Py_None;
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_Exception, e.what ());
+    }
+
+  return ret;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief remove folder
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -826,6 +922,12 @@ static PyMethodDef tp_methods[] =
     METH_VARARGS,
     "Check if file is browseable"
   },
+  {
+    (char *) "get_extension",
+    (PyCFunction) tp_f_get_extension,
+    METH_VARARGS,
+    "Get folder extension"
+  },
   {
     (char *) "get_parent",
     (PyCFunction) tp_f_get_parent,
@@ -862,6 +964,24 @@ static PyMethodDef tp_methods[] =
     METH_VARARGS,
     "Clear folder content"
   },
+  {
+    (char *) "reload",
+    (PyCFunction) tp_f_reload,
+    METH_VARARGS,
+    "Reload folder info"
+  },
+  {
+    (char *) "copy",
+    (PyCFunction) tp_f_copy,
+    METH_VARARGS,
+    "Copy folder"
+  },
+  {
+    (char *) "move",
+    (PyCFunction) tp_f_move,
+    METH_VARARGS,
+    "Move folder"
+  },
   {
     (char *) "remove",
     (PyCFunction) tp_f_remove,
